Attribute VB_Name = "modDesign"
Option Explicit
Option Compare Text

Private mForm As frmDesign

Public Function SaveRelations(tForm As frmDesign) As String
    
    Dim tXMLDoc As DOMDocument
    Dim tLabelNode As clsLabelNode
    
    If tForm.LabelNodes Is Nothing Then Exit Function
    If tForm.LabelNodes.Count = 0 Then Exit Function
    
    Set mForm = tForm
    Set tXMLDoc = NewDOMDocument("relations", True)
    
    
    '.. find rootnode (first)
    For Each tLabelNode In mForm.LabelNodes
      If tLabelNode.Parents Is Nothing Then
          SaveRelationsRecursive tXMLDoc.documentElement, tLabelNode
          Exit For
      ElseIf tLabelNode.Parents.Count = 0 Then
          SaveRelationsRecursive tXMLDoc.documentElement, tLabelNode
          Exit For
      End If
    Next
    
    Set mForm = Nothing
    SaveRelations = tXMLDoc.xml

End Function

Private Sub SaveRelationsRecursive(ByVal tNode As IXMLDOMNode, ByVal tLabelNode As clsLabelNode)

    Dim tChildLabelNode   As clsLabelNode
    Dim tChildNode        As IXMLDOMNode

    Set tChildNode = modXML.AddXMLElement(tNode, tLabelNode.CurLabel.Name)
    
    With tLabelNode
        
        modXML.AddXMLAttribute tChildNode, "name", .Key
        modXML.AddXMLAttribute tChildNode, "branchvalue", .BranchValue
        modXML.AddXMLAttribute tChildNode, "branchamount", .BranchAmount
        modXML.AddXMLAttribute tChildNode, "questionname", .QuestionName
        modXML.AddXMLAttribute tChildNode, "questiontype", .QuestionType
        modXML.AddXMLAttribute tChildNode, "reference", .Reference
        
        If Not tLabelNode.GlueParent Is Nothing Then modXML.AddXMLAttribute tChildNode, "glueparent", .GlueParent.Key
        If Not tLabelNode.GlueChild Is Nothing Then modXML.AddXMLAttribute tChildNode, "glueChild", .GlueChild.Key
        
        With .CurLabel
            modXML.AddXMLAttribute tChildNode, "caption", .Caption
            modXML.AddXMLAttribute tChildNode, "top", .Top
            modXML.AddXMLAttribute tChildNode, "left", .Left
            modXML.AddXMLAttribute tChildNode, "width", .Width
            modXML.AddXMLAttribute tChildNode, "height", .Height
        End With
    
    End With
    
    If Not tLabelNode.Children Is Nothing Then
        For Each tChildLabelNode In tLabelNode.Children
              SaveRelationsRecursive tChildNode, tChildLabelNode
        Next
    End If
    
End Sub
Public Sub RestoreRelations(ByVal tForm As frmDesign, ByVal XMLString As String)
    
    Dim tXMLDoc As DOMDocument
    Dim tNode As IXMLDOMNode
    
    Set tXMLDoc = New DOMDocument
    tXMLDoc.async = False
    
    Set mForm = tForm
    Set tXMLDoc = New DOMDocument
    
    If tXMLDoc.loadXML(XMLString) Then
        For Each tNode In tXMLDoc.documentElement.childNodes
            RestoreRelationsRecursive tNode
        Next
    End If
    
    Set mForm = Nothing
    
End Sub

Private Sub RestoreRelationsRecursive(ByVal tNode As IXMLDOMNode, Optional ByVal ParentLabelNode As clsLabelNode)

    Dim tChildNode        As IXMLDOMNode
    
    Dim tLine               As VB.Line
    Dim tLabel              As VB.Label
    Dim tLabelNode          As clsLabelNode
    Dim tChildLabelNode     As clsLabelNode
    Dim bChildrenAreAlreadyGenerated  As Boolean
    
    Dim tGlueParent         As String
    Dim tGlueChild          As String
    
    On Error Resume Next
    Set tLabelNode = mForm.LabelNodes(modXML.XMLAttrValue(tNode, "name"))
    If Err.Number <> 0 Then
        Set tLabelNode = New clsLabelNode
        Set tLabel = mForm.Controls.Add("VB.Label", modXML.XMLAttrValue(tNode, "name"))
    Else
        Set tLabel = tLabelNode.CurLabel
        bChildrenAreAlreadyGenerated = True
    End If
    On Error GoTo 0
    
    tGlueParent = modXML.XMLAttrValue(tNode, "glueparent")
    tGlueChild = modXML.XMLAttrValue(tNode, "gluechild")

    Set tLabelNode.ParentForm = mForm
    Set tLabelNode.CurLabel = tLabel
    Set tLabel.Container = mForm.picNodes
    
    tLabelNode.AddParent ParentLabelNode
    
    tLabelNode.BranchAmount = Val(modXML.XMLAttrValue(tNode, "branchamount"))
    tLabelNode.BranchValue = modXML.XMLAttrValue(tNode, "branchvalue")
    tLabelNode.QuestionName = modXML.XMLAttrValue(tNode, "questionname")
    tLabelNode.QuestionType = modXML.XMLAttrValue(tNode, "questiontype")
    tLabelNode.Reference = CBool(modXML.XMLAttrValue(tNode, "reference"))
    
    If tLabelNode.BranchAmount > 0 Then
      tLabel.BackColor = mForm.lblJako.BackColor
      tLabel.ForeColor = mForm.lblJako.ForeColor
    Else
      tLabel.BackColor = mForm.lblPerus.BackColor
      tLabel.ForeColor = mForm.lblPerus.ForeColor
    End If
    
    tLabel.BorderStyle = mForm.lblPerus.BorderStyle
    tLabel.Width = mForm.lblPerus.Width
    tLabel.Height = mForm.lblPerus.Height
    
    tLabel.Caption = modXML.XMLAttrValue(tNode, "caption")
    
    tLabelNode.Top = XMLAttrValue(tNode, "top")
    tLabelNode.Left = XMLAttrValue(tNode, "left")
    tLabelNode.Width = XMLAttrValue(tNode, "width")
    tLabelNode.Height = XMLAttrValue(tNode, "height")
    
    If Not ParentLabelNode Is Nothing Then
        If tGlueParent = ParentLabelNode.Key Then
            Set tLabelNode.GlueParent = ParentLabelNode
            Set ParentLabelNode.GlueChild = tLabelNode
            ParentLabelNode.AddChild tLabelNode
        Else
            Set tLine = mForm.Controls.Add("VB.Line", "Line" & mForm.Counter)
            ParentLabelNode.AddChild tLabelNode, tLine
        End If
    End If
               
    If Not bChildrenAreAlreadyGenerated Then
        mForm.LabelNodes.Add tLabelNode, tLabelNode.Key
        If tNode.childNodes.length > 0 Then
            For Each tChildNode In tNode.childNodes
                RestoreRelationsRecursive tChildNode, tLabelNode
            Next
        End If
    End If
    
    
End Sub
