Attribute VB_Name = "modXML"
Option Explicit
Option Compare Text
Dim mXMLDoc As DOMDocument

'Collect only needed tags
Public Function GetXMLTags(Node As IXMLDOMNode, Optional ByVal Tags As String = "", Optional ByVal OmitTags As String, Optional ByVal Intended As Boolean = True, Optional ByVal OrgText As Boolean = False, Optional OrgComment As Boolean = False) As DOMDocument
    Dim tBody   As IXMLDOMNode
    
    Set mXMLDoc = New DOMDocument
    mXMLDoc.appendChild mXMLDoc.createElement("HTML")
    GetXMLTagsRecursive Node, mXMLDoc.documentElement, Tags, OmitTags, Intended, OrgText, OrgComment
    Set GetXMLTags = mXMLDoc
    Set mXMLDoc = Nothing
    
End Function

Private Sub GetXMLTagsRecursive(Node As IXMLDOMNode, XMLNode As IXMLDOMNode, ByVal Tags As String, ByVal OmitTags As String, ByVal Intended As Boolean, ByVal OrgText As Boolean, OrgComment As Boolean)
  
  Dim ChildNode       As IXMLDOMNode
  Dim ChildAttribute  As IXMLDOMAttribute
  Dim ChildXMLNode    As IXMLDOMNode
  Dim tName           As String
  Dim bMatch          As Boolean
  Dim tValue          As String
  Dim i As Long
  Dim j As Long
  
  
  If InStr(1, Node.nodeName, "/") Then Debug.Assert True
  tName = UCase(Replace(Node.nodeName, "/", ""))
  
  '.. for eg. Script includes #text node
  '   but we don't need this text node
  If Len(OmitTags) Then
      If (InStr(1, "|" & OmitTags & "|", "|" & tName & "|") > 0) Then
          Exit Sub
      End If
  End If
  
  '... HTML root is always present
  If tName = "HTML" Then GoTo TagAttributes
  
  '.. check if matches to pattern
  '.. then we collect them to same level
  bMatch = True
  If Len(Tags) Then
      If (InStr(1, "|" & Tags & "|", "|" & tName & "|") = 0) Then
          bMatch = False
          GoTo TagAttributes
      End If
  End If
    
  '... to be continued
  
  If Node.nodeType = NODE_TEXT Then
      If Node.parentNode.nodeName = "script" Or OrgText Then
          Set ChildXMLNode = AddXMLText(XMLNode, Node.nodeValue)
      Else
          Set ChildXMLNode = AddXMLElement(XMLNode, "TEXTNODE", Node.nodeValue)
      End If
  ElseIf Node.nodeType = NODE_COMMENT Then
      If OrgComment Then
          Set ChildXMLNode = AddXMLComment(XMLNode, Node.nodeValue)
      Else
          Set ChildXMLNode = AddXMLElement(XMLNode, "COMMENTNODE", Node.nodeValue)
      End If
  Else
      Set ChildXMLNode = AddXMLElement(XMLNode, Node.nodeName, Node.nodeValue)
  End If
  
  
  If Not Node.Attributes Is Nothing Then
      For i = 0 To Node.Attributes.length - 1
          With Node.Attributes(i)
              If .nodeName = "style" Then
              ElseIf .specified Then
                  If Not IsNull(.nodeValue) Then XMLAttrValue(ChildXMLNode, LCase(.nodeName)) = .nodeValue
              ElseIf .nodeName = "value" Then
                  If Not IsNull(.nodeValue) Then
                    XMLAttrValue(ChildXMLNode, LCase(.nodeName)) = .nodeValue
                  End If
              End If
          End With
      Next
  End If
    
    
TagAttributes:

  '.. jos tag:issa on tavaraa niin kertn samalle tasolle kaikki
  '   matchaavat nodet.
  
      For i = 0 To Node.childNodes.length - 1
          If Intended And bMatch Then
              GetXMLTagsRecursive Node.childNodes(i), ChildXMLNode, Tags, OmitTags, Intended, OrgText, OrgComment
          Else
              GetXMLTagsRecursive Node.childNodes(i), XMLNode, Tags, OmitTags, Intended, OrgText, OrgComment
          End If
      Next
  
End Sub
'... create new dom document with processing instructions
Public Function NewDOMDocument(Optional ByVal RootName As String = "", Optional ByVal WithProcessingInstructions As Boolean = True, Optional ByVal RootValue As String) As DOMDocument
    
    Dim tDoc As DOMDocument
    Dim tPin As IXMLDOMProcessingInstruction
    
    Set tDoc = New DOMDocument
    tDoc.async = False
    
    If WithProcessingInstructions Then
      Set tPin = tDoc.createProcessingInstruction("xml", "version=""1.0"" encoding=""ISO-8859-1""")
      tDoc.appendChild tPin
    End If
    If Len(RootName) Then
      tDoc.appendChild tDoc.createElement(UCase(RootName))
      If Len(RootValue) Then
        tDoc.appendChild tDoc.createTextNode(RootValue)
      End If
    End If
    Set NewDOMDocument = tDoc
    
End Function
Public Function AddXMLAttribute(ByVal Node As IXMLDOMNode, ByVal Name As String, Optional ByVal Value As String = "") As IXMLDOMNode
    Dim tNode As IXMLDOMNode
    If Not Node Is Nothing Then
        Set tNode = Node.ownerDocument.createAttribute(LCase(Name))
        tNode.Text = Value
        Set AddXMLAttribute = Node.Attributes.setNamedItem(tNode)
    End If
End Function
Public Function AddXMLElement(ByVal Node As IXMLDOMNode, ByVal Name As String, Optional ByVal Value As Variant = "") As IXMLDOMNode
        
    Dim tNode As IXMLDOMNode
    Dim tText As IXMLDOMText
    If Not Node Is Nothing Then
        Set tNode = Node.ownerDocument.createElement(UCase(Name))
        If Not IsNull(Value) Then Set tText = AddXMLText(tNode, Value)
        Set AddXMLElement = Node.appendChild(tNode)
    End If

End Function
Public Function AddXMLText(ByVal Node As IXMLDOMNode, ByVal Value As String) As IXMLDOMNode
    Dim tNode As IXMLDOMNode
    If Not Node Is Nothing Then
        Set tNode = Node.ownerDocument.createTextNode(Value)
        Set tNode = Node.appendChild(tNode)
        Set AddXMLText = tNode
    End If
End Function
Public Function AddXMLComment(ByVal Node As IXMLDOMNode, ByVal Value As String) As IXMLDOMNode
    Dim tNode As IXMLDOMNode
    If Not Node Is Nothing Then
        Set tNode = Node.ownerDocument.createComment(Value)
        Set tNode = Node.appendChild(tNode)
        Set AddXMLComment = tNode
    End If
End Function
Public Property Get XMLAttrValue(ByVal Node As IXMLDOMNode, ByVal Name As String) As String
    Dim tAttr As IXMLDOMAttribute
    If Not Node Is Nothing Then
      If Not Node.Attributes Is Nothing Then
        Set tAttr = Node.Attributes.getNamedItem(LCase(Name))
        If Not tAttr Is Nothing Then XMLAttrValue = tAttr.Value
      End If
    End If
End Property
Public Property Let XMLAttrValue(ByVal Node As IXMLDOMNode, ByVal Name As String, ByVal Value As String)
    Dim tAttr As IXMLDOMAttribute
    If Not Node Is Nothing Then
        Set tAttr = Node.Attributes.getNamedItem(LCase(Name))
        If tAttr Is Nothing Then
          Set tAttr = Node.ownerDocument.createAttribute(LCase(Name))
          tAttr.Value = Value
          Node.Attributes.setNamedItem tAttr
        Else
          tAttr.Value = Value
        End If
    End If
End Property
Public Property Get XMLNodeValue(ByVal Node As IXMLDOMNode, ByVal Name As String) As String
    Dim tNode As IXMLDOMNode
    If Not Node Is Nothing Then
        Set tNode = Node.Attributes.getNamedItem(UCase(Name))
        If Not tNode Is Nothing Then XMLNodeValue = tNode.nodeValue
    End If
End Property
Public Property Let XMLNodeValue(ByVal Node As IXMLDOMNode, ByVal Name As String, ByVal Value As String)
    
    Dim tNode As IXMLDOMNode
    If Not Node Is Nothing Then
        Set tNode = Node.selectSingleNode(UCase(Name))
        If tNode Is Nothing Then
          Set tNode = Node.ownerDocument.createElement(UCase(Name))
          tNode.nodeValue = Value
          Node.appendChild tNode
        Else
          tNode.nodeValue = Value
        End If
    End If

End Property

